<?php
if (!defined('_PS_VERSION_'))
	exit;

class paystarpayment extends PaymentModule
{
	private $_html = '';
	private $_postErrors = array();

	public function __construct()
	{
		$this->name = 'paystarpayment';
		$this->tab = 'payments_gateways';
		$this->version = '1';
		$this->author = 'Shayan Kiyani';
		$this->bootstrap = true;

		$this->currencies = true;
  		$this->currencies_mode = 'checkbox';

		parent::__construct();
		require(dirname(__FILE__) . '/paystarconfig.php');
		$this->paystar_wsconfig = $ws_config;

		$this->page = basename(__FILE__, '.php');
		$this->displayName = $this->l($this->paystar_wsconfig['en_name'] . ' Payment');
		$this->description = $this->l('A free module to pay online for ' . $this->paystar_wsconfig['en_name'] . '.');
		$this->confirmUninstall = $this->l('Are you sure, you want to delete your details?');
		if (!sizeof(Currency::checkPaymentCurrencies($this->id)))
			$this->warning = $this->l('No currency has been set for this module');

		$config = Configuration::getMultiple(array('IPRESTA_PAYSTAR_PIN', ''));
		if (!isset($config['IPRESTA_PAYSTAR_PIN']))
			$this->warning = $this->l('Your ' . $this->paystar_wsconfig['en_name'] . ' Pin Code must be configured in order to use this module');
	}
	
	public function install()
	{
		if (!parent::install()
	    	OR !Configuration::updateValue('IPRESTA_PAYSTAR_PIN', '')
			OR !$this->registerHook('payment')
	      	OR !$this->registerHook('paymentReturn')){
			    return false;
		}
		return true;
	}
	
	public function uninstall()
	{
		if (!Configuration::deleteByName('IPRESTA_PAYSTAR_PIN')
			OR !parent::uninstall())
			return false;
		return true;
	}

	public function renderForm()
	{
		$fields_form = array(
			'form' => array(
				'legend' => array(
					'title' => $this->l('Settings'),
					'icon' => 'icon-cogs'
				),
				'input' => array(
					array(
						'type' => 'text',
						'label' => $this->l('Merchant Code'),
						'name' => 'IPRESTA_PAYSTAR_PIN',
						'class' => 'fixed-width-lg',
					),
				),
				'submit' => array(
					'title' => $this->l('Save'),
				)
			),
		);

		$helper = new HelperForm();
		$helper->show_toolbar = false;
		$helper->table =  $this->table;
		$lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
		$helper->default_form_language = $lang->id;
		$helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
		$helper->identifier = $this->identifier;
		$helper->submit_action = 'submitpaystar';
		$helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
		$helper->token = Tools::getAdminTokenLite('AdminModules');
		$helper->tpl_vars = array(
			'fields_value' => $this->getConfigFieldsValues(),
			'languages' => $this->context->controller->getLanguages(),
			'id_language' => $this->context->language->id
		);

		return $helper->generateForm(array($fields_form));
	}

	public function getConfigFieldsValues()
	{
		return array('IPRESTA_PAYSTAR_PIN' => Tools::getValue('IPRESTA_PAYSTAR_PIN', Configuration::get('IPRESTA_PAYSTAR_PIN')));
	}

	public function getContent()
	{
		$output = '';
		$errors = array();
		if (isset($_POST['submitpaystar']))
		{
			if (empty($_POST['IPRESTA_PAYSTAR_PIN']))
				$errors[] = $this->l('Your merchant code is required.');

			if (!count($errors))
			{
				Configuration::updateValue('IPRESTA_PAYSTAR_PIN', $_POST['IPRESTA_PAYSTAR_PIN']);
				$output = $this->displayConfirmation($this->l('Your settings have been updated.'));
			}
			else
				$output = $this->displayError(implode('<br />', $errors));
		}
		return $output.$this->renderForm();
	}

	public function prePayment()
	{
		$pin = Configuration::get('IPRESTA_PAYSTAR_PIN');
		$purchase_currency = new Currency(Currency::getIdByIsoCode('IRR'));
		$current_currency = new Currency($this->context->cookie->id_currency);
		if($current_currency->id == $purchase_currency->id)
			$amount = number_format($this->context->cart->getOrderTotal(true, 3), 0, '', '');
		else
			$amount = number_format($this->convertPriceFull($this->context->cart->getOrderTotal(true, 3), $current_currency, $purchase_currency), 0, '', '');

		try 
		{
			$description = "شناسه کالا : {$this->context->cart->id}";
			$description .= "\nشناسه کاربری خریدار : {$this->context->customer->id}";
			$description .= "\nنام و نام خانوادگی خریدار : {$this->context->customer->firstname} {$this->context->customer->lastname}";
			$description .= "\nایمیل خریدار : {$this->context->customer->email}";

			$f = array(
				'amount' => $amount/10,
				'pin' => trim($pin),
				'callback' => $this->context->link->getModuleLink('paystarpayment', 'validation'),
				'bank' => $this->paystar_wsconfig['dbank'],
				'description' => $description
			);

			$f_ = '';
			foreach($f as $k=>$v) 
				$f_ .= $k . '=' . urlencode($v) . '&'; 
				
			rtrim($f_, '&');

			$ch = curl_init();
			curl_setopt($ch,CURLOPT_URL, "http://{$this->paystar_wsconfig['address']}/api/create/");
			curl_setopt($ch,CURLOPT_POST, count($f));
			curl_setopt($ch,CURLOPT_POSTFIELDS, $f_);
			curl_setopt($ch,CURLOPT_RETURNTRANSFER, true);
			$res = curl_exec($ch);
			curl_close($ch);
			if (!$res) 
			{
				$this->context->smarty->assign(array('paystarname' => $this->paystar_wsconfig['en_name']));
				$this->context->controller->errors[] = $this->l('Could not connect to bank or service.');
				return false;				
			}
			else if(strlen($res) > 5)
			{		
				$this->context->cookie->__set("amount", (int)$amount);
				$this->context->smarty->assign(array(
					'redirect_link' => 'http://' . $this->paystar_wsconfig['address'] . '/startpay',
					'transid' => $res,
					'paystarname' => $this->paystar_wsconfig['en_name']
				));
				return true;					
			}
			else
			{
				$this->context->smarty->assign(array('paystarname' => $this->paystar_wsconfig['en_name']));
				$this->context->controller->errors[] = $this->l('Error!') . ' : '  . $res;
				return false;
			}				
		}
		catch(PrestaShopException $e)
		{
			$this->context->smarty->assign(array('paystarname' => $this->paystar_wsconfig['en_name']));
			$this->context->controller->errors[] = $this->l('Could not connect to bank or service.');
			return false;
		}			
	}

	public function verify($transid)
	{
		$pin = Configuration::get('IPRESTA_PAYSTAR_PIN');
		$purchase_currency = new Currency(Currency::getIdByIsoCode('IRR'));
		$current_currency = new Currency($this->context->cookie->id_currency);
		if($current_currency->id == $purchase_currency->id)
			$amount = number_format($this->context->cart->getOrderTotal(true, 3), 0, '', '');
		else
			$amount = number_format($this->convertPriceFull($this->context->cart->getOrderTotal(true, 3), $current_currency, $purchase_currency), 0, '', '');
		if((int)$amount != (int)$this->context->cookie->__get("amount"))
		{
			$this->context->controller->errors[] = $this->l('Payment amount is incorrect.');
			return false;
		}
		
		$res = false;
		try
		{
			$f = array(
				'amount' => $amount/10,
				'pin' => trim($pin),
				'transid' => $transid,
			);
			
			$f_ = '';
			foreach($f as $k=>$v) 
				$f_ .= $k . '=' . urlencode($v) . '&'; 
				
			rtrim($f_, '&');

			$ch = curl_init();
			curl_setopt($ch,CURLOPT_URL, "http://{$this->paystar_wsconfig['address']}/api/verify/");
			curl_setopt($ch,CURLOPT_POST, count($f));
			curl_setopt($ch,CURLOPT_POSTFIELDS, $f_);
			curl_setopt($ch,CURLOPT_RETURNTRANSFER, true);
			$res = curl_exec($ch);
			curl_close($ch);
		}
		catch(PrestaShopException $e)
		{
			$this->context->controller->errors[] = $this->l('Could not connect to bank or service.');
			return false;
		}		
		//$res = 0;
		if($res == 1)
			return true;
		else
		{
			if(empty($res))
				$res = 'تراکنش پرداخت نشده است .';
			
			$this->context->controller->errors[] = $this->l('Error!') . ' : '  . $res;
			return false;
		}
		return false;
	}

	public function hookPayment($params)
	{
		if(!$this->active)
			return ;

       $this->context->smarty->assign(array(
            'paystarname' => $this->paystar_wsconfig['en_name']
        ));	
	
		return $this->display(__FILE__, 'payment.tpl');
	}

	public function hookPaymentReturn($params)
	{
		if (!$this->active)
			return;

        $order = new Order(Tools::getValue('id_order'));

        $this->context->smarty->assign(array(
            'id_order' => Tools::getValue('id_order'),
			'reference' => $order->reference,
			'transid' => Tools::getValue('transid'),
            'ver' => $this->version
        ));

		return $this->display(__FILE__, 'confirmation.tpl');
	}
	
	/**
	 *
	 * @return float converted amount from a currency to an other currency
	 * @param float $amount
	 * @param Currency $currency_from if null we used the default currency
	 * @param Currency $currency_to if null we used the default currency
	 */
	public static function convertPriceFull($amount, Currency $currency_from = null, Currency $currency_to = null)
	{
		if ($currency_from === $currency_to)
			return $amount;
		if ($currency_from === null)
			$currency_from = new Currency(Configuration::get('PS_CURRENCY_DEFAULT'));
		if ($currency_to === null)
			$currency_to = new Currency(Configuration::get('PS_CURRENCY_DEFAULT'));
		if ($currency_from->id == Configuration::get('PS_CURRENCY_DEFAULT'))
			$amount *= $currency_to->conversion_rate;
		else
		{
            $conversion_rate = ($currency_from->conversion_rate == 0 ? 1 : $currency_from->conversion_rate);
			$amount = Tools::ps_round($amount / $conversion_rate, 2);
			$amount *= $currency_to->conversion_rate;
		}
		return Tools::ps_round($amount, 2);
	}
}